#Used to merge xpi's for firefox <= 2 and >= 3

use strict;

my $new_xpi_name = "merged.xpi";

my $temp_dir = "merge_dir";

mkdir $temp_dir or die "Unable to make $temp_dir";

my %unpack_dir = ();

my $mangle = "A";

for my $xpi (@ARGV)
  {
	 $unpack_dir{$xpi} = $mangle;

	 if (!($xpi =~ m/\.xpi$/))
		{
		  die "Expected list of xpi files, got $xpi";
		}
	 mkdir ("merge_dir/$unpack_dir{$xpi}") or die "Unable to make merge_dir/$unpack_dir{$xpi}";

	 system("unzip -d merge_dir/$unpack_dir{$xpi} $xpi") and die "Error unzipping $xpi";

	 $mangle++;
  }

chdir $temp_dir;

my $xpi_merge_dir = "merged_xpi";

my %install_data = ();

for my $xpi (@ARGV)
  {
	 #Guess I could use an xml or rdf parser....
	 $install_data{$xpi} = {};

	 open INSTALL_RDF, "<$unpack_dir{$xpi}/install.rdf";
	 while (<INSTALL_RDF>)
		{
		  if (m%<em:name>(.*)</em:name>%)
			 {
				$install_data{$xpi}{"name"} = $1;
			 }
		  if (m%<em:version>(.*)</em:version>%)
			 {
				$install_data{$xpi}{"version"} = $1;
			 }
		  if (m%<em:targetPlatform>(.*)</em:targetPlatform>%)
			 {
				$install_data{$xpi}{"platform"} = $1;
			 }
		  if (m%<em:minVersion>(.*)</em:minVersion>%)
			 {
				$install_data{$xpi}{"minVersion"} = $1;
			 }
		  if (m%<em:maxVersion>(.*)</em:maxVersion>%)
			 {
				$install_data{$xpi}{"maxVersion"} = $1;
			 }
		}
	 close INSTALL_RDF;
  }


#Validate that we can merge these xpis.
#Currently everything but version must match.

my @compare_list = @ARGV;
my $first_xpi = shift @compare_list;

my $name = $install_data{$first_xpi}{"name"};
my $version = $install_data{$first_xpi}{"version"};
my $platform = $install_data{$first_xpi}{"platform"};
my $min_target_version = $install_data{$first_xpi}{"minVersion"};
my $max_target_version = $install_data{$first_xpi}{"maxVersion"};

for my $xpi (@compare_list)
  {
	 if ($name ne $install_data{$xpi}{"name"})
		{
		  die "xpi names don't match $first_xpi:" . $install_data{$first_xpi}{"name"} . " $xpi:" . $install_data{$xpi}{"name"};
		}
	 if ($version ne $install_data{$xpi}{"version"})
		{
		  die "xpi versions don't match $first_xpi:" . $install_data{$first_xpi}{"version"} . " $xpi:" . $install_data{$xpi}{"version"};
		}
	 if ($platform ne $install_data{$xpi}{"platform"})
		{
		  die "xpi platforms don't match $first_xpi:" . $install_data{$first_xpi}{"platform"} . " $xpi:" . $install_data{$xpi}{"platform"};
		}

	 if ($min_target_version gt $install_data{$xpi}{"minVersion"})
		{
		  $min_target_version = $install_data{$xpi}{"minVersion"};
		}
	 if ($max_target_version lt $install_data{$xpi}{"maxVersion"})
		{
		  $max_target_version = $install_data{$xpi}{"maxVersion"};
		}
  }


print "\nBuilding new $name xpi for platform $platform and versions $min_target_version to $max_target_version\n\n";

my $lib_prefix;
my $lib_suffix;
if ($platform eq "WINNT_x86-msvc")
  {
	 $lib_prefix = "";
	 $lib_suffix = "dll";
  }
else
  {
	 $lib_prefix = "lib";
	 $lib_suffix = "so";
  }

#Build new xpi

mkdir $xpi_merge_dir or die "Unable to make $xpi_merge_dir";

my $mangle;

if ("1.5" eq $install_data{$first_xpi}{"minVersion"})
  {
	 $mangle = "_1_8";
  }
else
  {
	 $mangle = "_1_9";
  }

system ("cp -r $unpack_dir{$first_xpi}/chrome $xpi_merge_dir") and die "Unable to copy chrome";
system ("cp -r $unpack_dir{$first_xpi}/components $xpi_merge_dir") and die "Unable to copy components";
system ("cp -r $unpack_dir{$first_xpi}/cmgui_libraries $xpi_merge_dir") and die "Unable to copy cmgui_libraries";
system ("cp -r $unpack_dir{$first_xpi}/zinc_components $xpi_merge_dir/zinc_components$mangle") and die "Unable to copy zinc_components";

system ("cp -r $unpack_dir{$first_xpi}/plugins $xpi_merge_dir") and die "Unable to copy plugins";

for my $xpi (@compare_list)
  {
	 if ("1.5" eq $install_data{$xpi}{"minVersion"})
		{
		  $mangle = "_1_8";
		}
	 else
		{
		  $mangle = "_1_9";
		}

	 system ("cp -r $unpack_dir{$xpi}/zinc_components $xpi_merge_dir/zinc_components$mangle") and die "Unable to copy $xpi zinc_components";

  }


#Write merged install.rdf

open INSTALL_RDF, "<$unpack_dir{$first_xpi}/install.rdf";
open MERGED_INSTALL_RDF, ">$xpi_merge_dir/install.rdf";

while (<INSTALL_RDF>)
  {
	 s%<em:minVersion>(.*)</em:minVersion>%<em:minVersion>$min_target_version</em:minVersion>%;
	 s%<em:maxVersion>(.*)</em:maxVersion>%<em:maxVersion>$max_target_version</em:maxVersion>%;

	 print MERGED_INSTALL_RDF;
  }

close INSTALL_RDF;
close MERGED_INSTALL_RDF;

system ("cd $xpi_merge_dir; zip -ur ../../$new_xpi_name *") and die "Unable to zip $xpi_merge_dir into ../$new_xpi_name";

chdir ("..") or "Unable to chdir ..";
system ("rm -r $temp_dir") and die "Unable to clean up $temp_dir";
