#ifdef XP_WIN
#include <windows.h>
#endif
#include "zincPlugin.h"
#include "nsIProgrammingLanguage.h"

NS_IMPL_ISUPPORTS3(zincPlugin, zincIPlugin, zincIPluginWindow, nsIClassInfo)

nsPluginInstanceBase*
NS_NewPluginInstance(nsPluginCreateData* aCreateDataStruct)
{
  zincPlugin* mgpi = new zincPlugin(aCreateDataStruct->instance);
  NS_ADDREF(mgpi);
  int i;

  for (i = 0; i < aCreateDataStruct->argc; i++)
  {
#if defined (ZINC_DEBUG)
     printf("key: %s value: %s\n",aCreateDataStruct->argn[i], aCreateDataStruct->argv[i]);
#endif // defined (ZINC_DEBUG)

     // Windowless modes
    if (!strcmp(aCreateDataStruct->argn[i], "wmode") && !strcmp(aCreateDataStruct->argv[i], "transparent"))
    {
#if defined (ZINC_DEBUG)
      printf("Setting wmode transparent\n");
#endif // defined (ZINC_DEBUG)
      /* no window */
      NPN_SetValue(aCreateDataStruct->instance, NPPVpluginWindowBool, (void*)FALSE);
      /* transparent */
      NPN_SetValue(aCreateDataStruct->instance, NPPVpluginTransparentBool, (void*)TRUE);
    }

    if (!strcmp(aCreateDataStruct->argn[i], "wmode") && !strcmp(aCreateDataStruct->argv[i], "opaque"))
    {
#if defined (ZINC_DEBUG)
      printf("Setting wmode opaque\n");
#endif // defined (ZINC_DEBUG)
      /* no window */
      NPN_SetValue(aCreateDataStruct->instance, NPPVpluginWindowBool, (void*)FALSE);
      /* transparent */
      NPN_SetValue(aCreateDataStruct->instance, NPPVpluginTransparentBool, (void*)FALSE);
    }
  }

  return mgpi;
}

void
NS_DestroyPluginInstance(nsPluginInstanceBase* aPlugin)
{
	zincPlugin* gp = static_cast<zincPlugin*>(aPlugin);
	gp->shut();
	NS_RELEASE(gp);
}

NPError
NS_PluginInitialize()
{
  return NPERR_NO_ERROR;
}

void
NS_PluginShutdown()
{
}

char*
NPP_GetMIMEDescription()
{
  return "application/x-zinc-plugin::ZINC Plugin";
}


#ifdef XP_UNIX
NPError
NS_PluginGetValue(NPPVariable aVariable, void *aValue)
{
  if (aVariable == NPPVpluginNameString)
  {
    *((char **)aValue) = "CMISS Zinc Plugin";
    return NPERR_NO_ERROR;
  }
  else if (aVariable == NPPVpluginDescriptionString)
  {
    *((char**)aValue) = "A plugin that lets applications get a window handle.";
    return NPERR_NO_ERROR;
  }
  return NPERR_INVALID_PARAM;
}
#endif

zincPlugin::zincPlugin(NPP instance)
	: inited(FALSE), instance(instance)
{
	window = NULL;
	drawable = NULL;
}

NPBool
zincPlugin::init(NPWindow* aWindow)
{
  inited = TRUE;
  SetWindow(aWindow);
  return TRUE;
}

void
zincPlugin::shut()
{
#if defined (ZINC_DEBUG)
  printf("zincPlugin::shut:: Notifying resizeMonitor\n");
#endif // defined (ZINC_DEBUG)

  if (resizeMonitor != nsnull)
  {
    resizeMonitor->OnClose();
  }

  inited = FALSE;
  resizeMonitor = nsnull;
}

NPBool
zincPlugin::isInitialized()
{
  return inited;
}

NPError
zincPlugin::GetValue(NPPVariable variable, void *value)
{
  static nsIID scriptableIID = ZINCIPLUGIN_IID;
  nsIID* ptr;

  switch (variable)
  {
  case NPPVpluginNameString:
    *((char **)value) = "CMISS Zinc Plugin";
    return NPERR_NO_ERROR;
  case NPPVpluginDescriptionString:
    *((char **)value) = "CMISS Zinc Plugin";
    return NPERR_NO_ERROR;
  case NPPVpluginScriptableInstance:
    NS_ADDREF(this);
    *((nsISupports**)value) = static_cast<zincIPlugin*>(this);
    return NPERR_NO_ERROR;
  case NPPVpluginScriptableIID:
    ptr = (nsIID*)NPN_MemAlloc(sizeof(nsIID));
    if (ptr)
    {
      *ptr = scriptableIID;
      *(nsIID **)value = ptr;
    }
    else
      return NPERR_OUT_OF_MEMORY_ERROR;
    return NPERR_NO_ERROR;
  case NPPVpluginNeedsXEmbed:
    *(int*)value = 1;
    return NPERR_NO_ERROR;
  }
  return NPERR_INVALID_PARAM;
}

NPError
zincPlugin::SetWindow(NPWindow* aWindow)
{
	switch (aWindow->type)
	{
		case NPWindowTypeWindow:
		{
			window = aWindow->window;
		} break;
		case NPWindowTypeDrawable:
		{
			drawable = aWindow->window;
		} break;
	}
	width = aWindow->width;
	height = aWindow->height;
	x = aWindow->x;
	y = aWindow->y;

#if defined (ZINC_DEBUG)
	printf("zincPlugin::SetWindow called window:%p drawable:%p\n",
		window, drawable);
	printf("zincPlugin::SetWindow x %d, y %d\n",
		x, y);
#endif // defined (ZINC_DEBUG)

	if (resizeMonitor != nsnull)
	{
		resizeMonitor->OnResize(width, height, x, y);
	}
	return NPERR_NO_ERROR;
}


NS_IMETHODIMP
zincPlugin::GetWidth(PRUint32* owidth)
{
  *owidth = width;
  return NS_OK;
}

NS_IMETHODIMP
zincPlugin::GetHeight(PRUint32* oheight)
{
  *oheight = height;
  return NS_OK;
}


NS_IMETHODIMP
zincPlugin::GetFlags(PRUint32 *aFlags)
{
  *aFlags = nsIClassInfo::PLUGIN_OBJECT | nsIClassInfo::DOM_OBJECT;
  return NS_OK;
}

NS_IMETHODIMP
zincPlugin::GetImplementationLanguage(PRUint32 *aImplementationLanguage)
{
  *aImplementationLanguage = nsIProgrammingLanguage::CPLUSPLUS;
  return NS_OK;
}

NS_IMETHODIMP
zincPlugin::GetInterfaces(PRUint32 *count, nsIID * **array)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
zincPlugin::GetHelperForLanguage(PRUint32 language, nsISupports **_retval)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
zincPlugin::GetContractID(char * *aContractID)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
zincPlugin::GetClassDescription(char * *aClassDescription)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
zincPlugin::GetClassID(nsCID * *aClassID)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
zincPlugin::GetClassIDNoAlloc(nsCID *aClassIDNoAlloc)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
zincPlugin::GetResizeMonitor(zincIPluginResizeMonitor** rm)
{
  *rm = resizeMonitor;
  if (*rm != nsnull)
    NS_ADDREF(*rm);
  return NS_OK;
}

NS_IMETHODIMP
zincPlugin::SetResizeMonitor(zincIPluginResizeMonitor* rm)
{
  resizeMonitor = rm;
  return NS_OK;
}

uint16
zincPlugin::HandleEvent(void* aEvent)
{
#if defined (ZINC_DEBUG)
	//printf("Entered: zincPlugin::HandleEvent\n");
#endif // defined (ZINC_DEBUG)

  if (resizeMonitor != nsnull)
  {
    resizeMonitor->OnEvent(aEvent);
  }

  return NS_OK;
}

/* void repaintRequired (); */
NS_IMETHODIMP zincPlugin::RepaintRequired()
{
#if defined (ZINC_DEBUG)
	printf("zincPlugin::RepaintRequired instance:%p\n",
		instance);
#endif // defined (ZINC_DEBUG)

	NPRect wholePluginRect;
 	wholePluginRect.top = 0;
 	wholePluginRect.left = 0;
 	wholePluginRect.bottom = height;
 	wholePluginRect.right = width;
	NPN_InvalidateRect(instance, &wholePluginRect);
	NPN_ForceRedraw(instance);
	return NS_OK;
}
